function [y, lik_mat, pr_st0, pr_stl0] = lik_ms_sbvar_svd(xdd, F, impact, SDX, Q, nstates)
%
% Compute the sample log-likelihood function and the filtered probabilities
% of a Markov-switching reduced-form VAR(1) model.
%
% The model takes the form:  x(t) = G* x(t-1) + R* e(t)
%                            e(t) ~ N(0,SDX'*SDX)  
%                            H    = R* SDX
% 
% "MAKING MACRO MODELS BEHAVE REASONABLY" BY CHRIS SIMS (2008):
% "H is likely to be of less than full row rank, implying singularity of the 
% covariance matrix. Therefore to write the log-pdf element for y corresponding
% to the dummy observation we can use the singular value decomposition H = UDV0,
% where V'*V = U'*U = I and D is a square diagonal matrix with no zeros on
% the diagonal":
%                 -log|D| - 0.5*[x(t)-G*x(t-1)]'*U*D^(-2)*U'*[x(t)-G*x(t-1)]
% where H = U*D*V', V'*V = U*U' = I and D is a square diagonal matrix with
% no zeros on the diagonal.
%
% Stephane Lhuissier 2015

t    = size(xdd,1);
nvar = size(xdd,2);

yy   = xdd(2:end,:)';
xx   = xdd(1:end-1,:)';

% Initializing Markov-switching framework
%--------------------------------------------------------------
%Steady-state probabilities
phi_ss     = [eye(nstates) - Q; ones(1,nstates)]; 
y_ss       = [zeros(nstates,1);1];
Prob_ss = fn_ols(y_ss,phi_ss); % see Kim and Nelson (1999), page 70, section 4.3.2
Prob    = vecr(Prob_ss(:,ones(1,nstates)));

pr_st0  = zeros(t-1,nstates); % save all Pr[S_t=0|Y_{t}]
pr_stl0 = zeros(t-1,nstates); % save all Pr[S_t=0|Y_{t-1}]

% The sample log-likelihood function
%--------------------------------------------------------------
lik_mat = zeros(t-1,1);
lcond   = zeros(nstates,1);
fss     = -0.5*nvar*log(2*pi);

[m,n] = size(impact);

detS = zeros(nstates,1);
H    = zeros(m,n,nstates);
U    = zeros(m,n,nstates);
S    = zeros(n,n,nstates);
V    = zeros(n,n,nstates);

for j_iter=1:nstates
    H(:,:,j_iter) = impact*SDX(:,:,j_iter);
    %
    [U(:,:,j_iter),S(:,:,j_iter),V(:,:,j_iter)] = svd(H(:,:,j_iter),0);
    [~,Su]           = lu(S(:,:,j_iter));
    detS(j_iter,1)   = sum(log(abs(diag(Su))));
end


for k=1:t-1

    Prob_j = Q(:).*Prob; % A 4x1 vector  
 
    for j=1:nstates
        % conditional log-likelihood
	    lcond(j,:) = -detS(j,1) - 0.5*((yy(:,k)-F*xx(:,k))'*(U(:,:,j)*S(:,:,j)^(-2)*U(:,:,j)')*((yy(:,k)-F*xx(:,k))));
    end

	vec_lcond = lcond(:,ones(1,nstates));

	%conditional densities, weight being Pr[S_{t}=j,S_{t-1}=i|Y_{t-1}]
	densj = fss + vec_lcond(:) + log(Prob_j);


	%log of Weighted average of conditional densities Pr[Y_t|Y_{t-1}] --
	maxdensj = max(densj);
	dens = log(sum(exp(densj - maxdensj))) + maxdensj;

	%log likelihood for each date
	lik_mat(k,1) = dens;

	%Update the probability terms Pr[S_t,S_{t-1}|Y_t] -----------
	JointProb = exp(densj - dens);

	%Update the probability terms Pr[S_t|Y_t] -------------------
	Pr = sum(reshape(JointProb,nstates,nstates),2);

	%build the probability terms Prob for the next period
	Prob = vecr(Pr(:,ones(1,nstates)));

	%get the probability terms Pr[S_t=0|Y_{t-1}] for Kim filter
	tmp = sum(reshape(Prob_j,nstates,nstates),2);

	pr_st0(k,:)  = Pr';  % Pr[S_t=0|Y_t]
	pr_stl0(k,:) = tmp'; % Pr[S_t=0|Y_{t-1}]

end

pr_st0 = [Prob_ss'; pr_st0]; % adding the unconditional probabilities at date 0 
y = sum(lik_mat(:,1));
