% Main file for estimating a MS-DSGE model 
%
% Stephane Lhuissier, 2015

clear all
clc

global bayestopt_

addpath 'Library'
 
% Input and Data
%--------------------------------------------------------------------------
% number of regimes
nstates=2;

% load data
load data_eadsge_Lhuissier.prn
options_.data = data_eadsge_Lhuissier;

% first year to use in estimation
yrStart=1970;
% first quarter or month to use in estimation
qmStart=2;
% last year to use in estimation
yrEnd=2012;
% last quater or month to use in estimation
qmEnd=1;

options_.nstates = nstates;
%--------------------------------------------------------------------------
% create date 
[date, n] = fn_calyrqm(4, [yrStart qmStart], [yrEnd qmEnd]);
qdates=zeros(n,1);
for ki=1:n
   qdates(ki) = date(1,1) + (date(1,2)+ki-2)/4;
end
%--------------------------------------------------------------------------
options_.name_lik   = 'ms_lik_fc';
%--------------------------------------------------------------------------

% Priors
%--------------------------------------------------------------------------
eval(['R',num2str(nstates),'v_prior']);
options_.pshape   = prior(:,1);
options_.sp       = prior(:,2);
options_.pmean    = prior(:,3);
options_.pstdd    = prior(:,4);
options_.lb       = prior(:,5);
options_.ub       = prior(:,6);
options_.mask     = prior(:,7);
options_.maskinv  = 1-options_.mask;
options_.fix      = prior(:,8);
options_.block    = prior(:,9);
bayestopt_.penalty  = 1E8; 

% Input for C.S optimizer
%--------------------------------------------------------------------------
options_.nparam  = size(options_.sp,1);
options_.imstp   = 4*4; % impulses reponses


% Position entries calibrated
poscal           = find(prior(:,7)); 
options_.param   = options_.sp;
options_.posest  = (1:length(options_.param)); 
options_.posest  = find( ismember(options_.posest(:),poscal(:)) == 0 ); 
x0               = options_.param(options_.posest);
options_.lb      = options_.lb(options_.posest);
options_.ub      = options_.ub(options_.posest);

H0               = 1e-4*eye(length(x0));
crit             = 1.0e-6;
nit              = 10000;
gradient_method  = 2;
gradient_epsilon = 1e-6;

number_of_perturbations = 1;
options_.post           = 1; % MLE (0) or Posterior mode (1)
options_.blockwise      = 0; % 

% Numerical optimization by Christopher Sims : BFGS optimizer
%--------------------------------------------------------------------------
tic
%load R2v_posterior_mode
%	x0 = xh;
[fh,xh,gh,H,itct,fcount,retcodeh] = csminwel1('logpostMSDSGE',x0,1e-4*eye(length(x0)),[],crit,nit, ...
                                       gradient_method, gradient_epsilon, options_, 1)
                                   
% display computing time	
fprintf(1,'Total computing time: %s\n',secs2hms(toc));	

% Output results
%--------------------------------------------------------------------------
% Transform posterior from constrained minimization space to parameter
% space 
postmode=options_.param;
postmode(options_.posest)=xh;

% Account for the transformation when computing Hessian 
HH=0.5*(H+H'); 

% evaluate the prior value at the posterior mode
lnprio = priodens(postmode, options_.pmean, options_.pstdd, options_.pshape);

% evaluate the likelihood value at the posterior mode
[lnlik, lik_mat, pr_st0, pr_stl0, Q] = ms_lik_fc(postmode, options_);

% Kim (1994) filter 
[pr_sT, pr_joint_sT] = SmoothedProbabilities(pr_st0, pr_stl0, Q);

% save posterior mode
eval(['save R', num2str(nstates), 'v_posterior_mode postmode xh HH x0 gh lnlik lnprio pr_st0 pr_stl0 pr_sT'])

% display posterior mode 
param_name = {'alpha';'delta';'iotap';'iotaw'; 'gammastar100';'gammamiu100'; ...
              'h';'lambdapsi';'lambdaw';'Lss'; 'pss100';'Fbeta';'gss';'niu'; ...
              'xip';'xiw';'chi';'S''''';'fp';'fy';'rhoR';'rhoz';'rhog';'rhomiu'; ...
			  'rholambdap';'rhopsi';'rhob';'rhoARMAlambdap';'rhoARMApsi';    ...
			  'fdy';'rhomp';'rhoupsilon';'flag_relpiobs'};
          
std_name = cell(nstates,1);
for i_iter=1:nstates
    std_name{i_iter}={ 
     ['sdr(st=',  num2str(i_iter), ')']; ['sdz(st=',  num2str(i_iter), ')']; ...
     ['sdg(st=',  num2str(i_iter), ')']; ['sdmiu(st=',  num2str(i_iter), ')']; ...
     ['sdlambda(st=',  num2str(i_iter), ')']; ['sdpsi(st=',  num2str(i_iter), ')']; ...
     ['sdb(st=',  num2str(i_iter), ')']; ['sdups(st=',  num2str(i_iter), ')']};
end

pr_name = cell(nstates,1);
for i_iter=1:nstates
    pr_name{i_iter}=['p',  num2str(i_iter),num2str(i_iter)]; 
end

nfp = size(options_.param,1)-nstates-1-length(std_name{1})*nstates;
for i=1:nfp
    fprintf('%14s   %1.4f\n',param_name{i},postmode(i))
end
uu=0;
for i=1:nstates
    for j_iter=1:length(std_name{1})
        uu = uu + 1;
        fprintf('%14s   %1.4f\n',std_name{i}{j_iter},postmode(nfp+uu))
    end
end
for i=1:nstates
    fprintf('%14s   %1.4f\n',pr_name{i},postmode(size(options_.param,1)-nstates-1+i))
end
fprintf('%14s   %1.4f\n','nstates',postmode(end))


figure
area(qdates,(pr_sT(:,1)))
ylim([0 1])
title('CHAIN I')




