% Main file for simulating a MS-DSGE model
%
% Stephane Lhuissier, 2015

clear all; clc;

addpath 'Library'
 
% =========================================================================
% Data and options
% =========================================================================
% number of draws
ndraws = 10000;
% number of regimes
nstates = 2;
% types of model
other_specifications = '';
% index for shock variances
idx_specification = [1 1 1 1 1 1 1 1];
% scaling for RWMH 
cc = 0.40; 
% number of chains
nchains = 1;
% thinning factor
thinning_factor = 10;
% log-likelihood computed with Kalman filter starts at date t0+1
t0 = 4;
% criteria value for pinv in the function "gen_zt.m"
cv = 1E-3;
% number of clusters
idx_cluster = 1;
% compute the Hessian matrix
IDX_HESSIAN = 0; 
% seed number for each chain
seednumber = [756; 345; 582];
% M_{jj} as in SWZ (2008, JoE)
M_cell{1} = [1 0; 
             0 1]; 
M_cell{2} = [0 1; 
             1 0];
%
randn('state',seednumber(idx_cluster));
rand('state',seednumber(idx_cluster));
% =========================================================================
% load data
load data_eadsge_Lhuissier.prn
xdd = data_eadsge_Lhuissier;
clear data_eadsge_Lhuissier
% first year to use in estimation
yrStart=1970;
% first quarter or month to use in estimation
qmStart=2;
% last year to use in estimation
yrEnd=2012;
% last quater or month to use in estimation
qmEnd=1;
% size of the sample
t=size(xdd,1);
% =========================================================================
eval(['R',num2str(nstates),'v',other_specifications,'_prior']);
options_.pshape   = prior(:,1);
options_.sp       = prior(:,2);
options_.pmean    = prior(:,3);
options_.pstdd    = prior(:,4);
options_.lb       = prior(:,5);
options_.ub       = prior(:,6);
options_.mask     = prior(:,7);
options_.maskinv  = 1-options_.mask;
options_.fix      = prior(:,8);
options_.block    = prior(:,9);

% load posterior mode and all options 
eval(['load R', num2str(nstates), 'v',other_specifications,'_posterior_mode'])

% Position entries calibrated
poscal           = find(options_.mask); 
options_.param   = options_.sp;
options_.posest  = (1:length(options_.param)); 
options_.posest  = find( ismember(options_.posest(:),poscal(:)) == 0 ); 

% Position entries calibrated - DSGE structural parameters (theta)
poscal            = find(options_.block~=1); 
options_.postheta = (1:length(options_.param)); 
options_.postheta = find( ismember(options_.postheta(:),poscal(:)) == 0 ); 
options_.nparam   = size(options_.postheta,1);

% Position entries calibrated - structural innovations (sig)
poscal           = find(options_.block~=2); 
options_.possig  = (1:length(options_.param)); 
options_.possig  = find( ismember(options_.possig(:),poscal(:)) == 0 ); 

% Position entries calibrated - transition probabilities (q)
poscal         = find(options_.block~=3); 
options_.posq  = (1:length(options_.param)); 
options_.posq  = find( ismember(options_.posq(:),poscal(:)) == 0 ); 

% =========================================================================
% INITIAL VALUES FOR GIBBS SAMPLER (AT MODE)
% =========================================================================
disp('Obtaining starting values ...');

% Each parameter at its mode
postmode                   = options_.param;              % at initial value of CSMINWEL
postmode(options_.posest)  = xh;                          % at posterior mode
postmode_DSGEstruct        = postmode(options_.postheta); % DSGE structural parameters
postmode_sig               = (postmode(options_.possig)); % structural shocks

% Solution of the JPT model 
[F,CC,impact,eu,SDX,P,H,NY,NX,NETA,gev,ncof,PSI] = eval(['modptj',num2str(nstates),'v',other_specifications,'(postmode)']);

% Kim and Nelson (1999) filter
[~, ~, pr_st0, ~] = kn_filter(xdd, F, impact, SDX, P, H, CC, nstates);

% ST at the mode
[~, STmode] = max(pr_st0,[],2);

if IDX_HESSIAN
    % compute Hessian matrix
    hessian = hessizero('ms_post_fc_conditional_path',[postmode_DSGEstruct, zeros(size(postmode_DSGEstruct,1))],1,options_.param, options_.postheta, options_.possig, options_.posq, xdd, STmode, postmode_sig, other_specifications, idx_specification, options_);	

    % Create Inverse by Singular Value Decomposition
    [u,s,v]  = svd(hessian);
    md = min(size(s)); 
    bigev = find(diag(s(1:md,1:md))>1e-12);
    sigpropdim = length(bigev);

    sigproplndet = 0;
    for i = 1:options_.nparam
        if i > sigpropdim
            s(i,i) = 0;
        else
            s(i,i)    = 1/s(i,i);
            sigproplndet = sigproplndet+log(s(i,i));  
        end
    end
    sigscale = u*sqrt(s);

    % save Hessian
    eval(['save R', num2str(nstates), 'v',other_specifications,'_hessian hessian sigscale'])
else
    % load Hessian
    eval(['load R', num2str(nstates), 'v',other_specifications,'_hessian'])
end

% % Generate unobservable variable ST
ST=zeros(t,1);
for ii=1:t
    ST(ii,1) = bingen1(pr_st0(ii,:));
end

% Evaluate DSGE state vector at the mode
Q = zeros(NY,NY,nstates);
for ii=1:nstates
    Q(:,:,ii) = impact*(SDX(:,:,ii)^2)*impact';
end
[~, stateNEW] = kf_conditional_path(xdd, F, Q, H, H*CC, 0, ST);
stateNEW=stateNEW';

% Initilization of RWMH
drawsMat=zeros(size(options_.postheta,1),ndraws);

logpostOLD=-1e10;
    while logpostOLD==-1e10;
        xguess = postmode_DSGEstruct + cc*(sigscale*randn(options_.nparam,1));
        % check bounds
        ind1 = find(xguess > options_.ub(options_.postheta));
        ind2 = find(xguess < options_.lb(options_.postheta));
        if any(ind1) || any(ind2)
            logpostOLD = -1e10;
        else
        logpostOLD = ms_post_fc_conditional_path(xguess, options_.param, options_.postheta, options_.possig, options_.posq, xdd, ST, postmode_sig, other_specifications, idx_specification, options_);
        valid0 = 1
        end
    end 
    drawsMat(:,1)=xguess; 
%end 

% Initial values for DSGE parameters (structural, shock processes,
% transition probabilities)
paramNEW = postmode;
sigNEW   = postmode_sig;
qNEW     = P;

paramNEW(options_.postheta) = xguess';

% Justiniano, Primiceri and Tambalotti (2011, RED)  with MS
[F,CC,impact,eu,SDX,P,H,NY,NX,NETA] = eval(['modptj',num2str(nstates),'v',other_specifications,'(paramNEW)']);

% =========================================================================
% GIBBS SAMPLING.
% =========================================================================
currentFile00 = strcat('SIMULATION_R',num2str(nstates),'v/');

if  exist(strcat('SIMULATION_R',num2str(nstates),'v/'),'file') == 0
    mkdir(strcat('SIMULATION_R',num2str(nstates),'v/'))
end

tic
for j_iter=1:nchains
    
% memory allocation of draws
nreplic    = ndraws/thinning_factor;
rej        = zeros(ndraws,1);
STgibbs    = zeros(t,nreplic);
Qgibbs     = zeros(nstates,nstates,nreplic);
DSGEgibbs  = zeros(t,NY,nreplic);
SIGgibbs   = zeros(size(SDX,1),nstates,nreplic);
THETAgibbs = zeros(size(drawsMat,1),nreplic);

% memory allocation of initial draw
STgibbs(:,1)     = ST;
Qgibbs(:,:,1)    = qNEW;
DSGEgibbs(:,:,1) = stateNEW;
THETAgibbs(:,1)  = drawsMat(:,1);


xxx = [ones(1,NX);repmat(idx_specification,nstates-1,1)];
sxx = find(xxx');
sigNEW  = NaN*ones(NX,nstates);
sigNEW(sxx)  = postmode_sig;

jj = 1;
for i_iter=2:ndraws 
    
    % DISPLAY PROGRESS
    % ---------------------------------------------------------------------
    fprintf(1,'\n[cluster no. %d]',idx_cluster);
    
    % STEP 1. generate unobservable states, st
    % ---------------------------------------------------------------------
    ST = gen_st(nstates, stateNEW, F, impact, SDX, qNEW);

    % STEP 2. generate transition matrix, Q
    % ---------------------------------------------------------------------
    qNEW = gen_q1(options_.pmean(options_.posq),options_.pstdd(options_.posq),ST,M_cell,nstates);
    
    % STEP 3. generate DSGE structural parameters, theta
    % ---------------------------------------------------------------------   
    [drawsMat, rej] = gen_mu(drawsMat, i_iter, xdd, paramNEW, ...
                    options_.postheta, sigscale, cc, ndraws, options_.lb, options_.ub, ST, sigNEW, ...
                    options_.pmean, options_.pstdd, options_.pshape, rej, options_.nparam, j_iter, other_specifications, idx_specification, options_);            
    % 
    paramNEW(options_.postheta) = drawsMat(:,i_iter);
    
    % STEP 4. generate DSGE state vector, zt
    % ---------------------------------------------------------------------
    % Justiniano, Primiceri and Tambalotti (2011, RED)  with MS
    [F,CC,impact,eu,SDX,P,H] = eval(['modptj',num2str(nstates),'v(paramNEW)']);
    % Generate zt
    stateNEW = gen_zt(xdd, F, H, 0, impact, SDX, H*CC, ST, nstates, cv);

    % STEP 5. generate structural shocks, sigma
    % ---------------------------------------------------------------------
    sigNEW = gen_sig1(ST, stateNEW, F, impact, options_.pmean(options_.possig), options_.pstdd(options_.possig), nstates, idx_specification);
    for v_iter=1:nstates
        SDX(1:NX,1:NX,v_iter)=diag(sigNEW(:,v_iter)); 
    end
    
    % ALLOCATE NEW VALUES
    % ---------------------------------------------------------------------
    paramNEW(options_.possig) = sigNEW(:);
    paramNEW(options_.posq)   = diag(qNEW); 
    
        
    % STORE DRAWS [keep only every (thinning_factor)th draw]
    % ---------------------------------------------------------------------
    if (i_iter / thinning_factor) == fix(i_iter / thinning_factor)
        STgibbs(:,jj)     = ST;
        Qgibbs(:,:,jj)    = qNEW;
        DSGEgibbs(:,:,jj) = stateNEW;
        SIGgibbs(:,:,jj)  = sigNEW;
        THETAgibbs(:,jj)  = drawsMat(:,i_iter);
        jj = jj + 1;
    end
end

% save simulation
eval(['save SIMULATION_R', num2str(nstates), 'v/R', num2str(nstates),'v_simulation_cluster_',num2str(idx_cluster), '_block_',num2str(j_iter), ' rej STgibbs Qgibbs DSGEgibbs SIGgibbs THETAgibbs'])

end

fprintf(1,'\nTotal computing time: %s\n',secs2hms(toc));
